#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Terrain Analysis
- Name     : Profiles
- ID       : ta_profiles

Description
----------
Simple, flow path and swath profiles.
'''

from PySAGA.helper import Tool_Wrapper

def Cross_Profiles(
    DEM=None,
    LINES=None,
    PROFILES=None,
    DIST_LINE=None,
    DIST_PROFILE=None,
    NUM_PROFILE=None,
    INTERPOLATION=None,
    OUTPUT=None, Verbose=2):
    '''
    Cross Profiles
    ----------
    [ta_profiles.3]\n
    Create cross profiles from a grid based DEM for given lines.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - LINES [`input shapes`] : Lines
    - PROFILES [`output shapes`] : Cross Profiles
    - DIST_LINE [`floating point number`] : Profile Distance. Minimum: 0.000000 Default: 10.000000 The distance of each cross profile along the lines.
    - DIST_PROFILE [`floating point number`] : Profile Length. Minimum: 0.000000 Default: 10.000000 The length of each cross profile.
    - NUM_PROFILE [`integer number`] : Profile Samples. Minimum: 3 Default: 11 The number of profile points per cross profile.
    - INTERPOLATION [`choice`] : Interpolation. Available Choices: [0] Nearest Neighbour [1] Bilinear Interpolation [2] Bicubic Spline Interpolation [3] B-Spline Interpolation Default: 3
    - OUTPUT [`choice`] : Output. Available Choices: [0] vertices [1] attributes [2] vertices and attributes Default: 2

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_profiles', '3', 'Cross Profiles')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('LINES', LINES)
        Tool.Set_Output('PROFILES', PROFILES)
        Tool.Set_Option('DIST_LINE', DIST_LINE)
        Tool.Set_Option('DIST_PROFILE', DIST_PROFILE)
        Tool.Set_Option('NUM_PROFILE', NUM_PROFILE)
        Tool.Set_Option('INTERPOLATION', INTERPOLATION)
        Tool.Set_Option('OUTPUT', OUTPUT)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_profiles_3(
    DEM=None,
    LINES=None,
    PROFILES=None,
    DIST_LINE=None,
    DIST_PROFILE=None,
    NUM_PROFILE=None,
    INTERPOLATION=None,
    OUTPUT=None, Verbose=2):
    '''
    Cross Profiles
    ----------
    [ta_profiles.3]\n
    Create cross profiles from a grid based DEM for given lines.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - LINES [`input shapes`] : Lines
    - PROFILES [`output shapes`] : Cross Profiles
    - DIST_LINE [`floating point number`] : Profile Distance. Minimum: 0.000000 Default: 10.000000 The distance of each cross profile along the lines.
    - DIST_PROFILE [`floating point number`] : Profile Length. Minimum: 0.000000 Default: 10.000000 The length of each cross profile.
    - NUM_PROFILE [`integer number`] : Profile Samples. Minimum: 3 Default: 11 The number of profile points per cross profile.
    - INTERPOLATION [`choice`] : Interpolation. Available Choices: [0] Nearest Neighbour [1] Bilinear Interpolation [2] Bicubic Spline Interpolation [3] B-Spline Interpolation Default: 3
    - OUTPUT [`choice`] : Output. Available Choices: [0] vertices [1] attributes [2] vertices and attributes Default: 2

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_profiles', '3', 'Cross Profiles')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('LINES', LINES)
        Tool.Set_Output('PROFILES', PROFILES)
        Tool.Set_Option('DIST_LINE', DIST_LINE)
        Tool.Set_Option('DIST_PROFILE', DIST_PROFILE)
        Tool.Set_Option('NUM_PROFILE', NUM_PROFILE)
        Tool.Set_Option('INTERPOLATION', INTERPOLATION)
        Tool.Set_Option('OUTPUT', OUTPUT)
        return Tool.Execute(Verbose)
    return False

def Profiles_from_Lines(
    DEM=None,
    LINES=None,
    VALUES=None,
    PROFILE=None,
    PROFILES=None,
    NAME=None,
    SPLIT=None, Verbose=2):
    '''
    Profiles from Lines
    ----------
    [ta_profiles.4]\n
    Create profiles from a grid based DEM for each line of a lines layer.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - LINES [`input shapes`] : Lines
    - VALUES [`optional input grid list`] : Values. Additional values to be collected along profile.
    - PROFILE [`output shapes`] : Profiles
    - PROFILES [`output shapes list`] : Profiles
    - NAME [`table field`] : Name. Attribute to use for split line naming (=> each line as new profile)
    - SPLIT [`boolean`] : Each Line as new Profile. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_profiles', '4', 'Profiles from Lines')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('LINES', LINES)
        Tool.Set_Input ('VALUES', VALUES)
        Tool.Set_Output('PROFILE', PROFILE)
        Tool.Set_Output('PROFILES', PROFILES)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('SPLIT', SPLIT)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_profiles_4(
    DEM=None,
    LINES=None,
    VALUES=None,
    PROFILE=None,
    PROFILES=None,
    NAME=None,
    SPLIT=None, Verbose=2):
    '''
    Profiles from Lines
    ----------
    [ta_profiles.4]\n
    Create profiles from a grid based DEM for each line of a lines layer.\n
    Arguments
    ----------
    - DEM [`input grid`] : Elevation
    - LINES [`input shapes`] : Lines
    - VALUES [`optional input grid list`] : Values. Additional values to be collected along profile.
    - PROFILE [`output shapes`] : Profiles
    - PROFILES [`output shapes list`] : Profiles
    - NAME [`table field`] : Name. Attribute to use for split line naming (=> each line as new profile)
    - SPLIT [`boolean`] : Each Line as new Profile. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_profiles', '4', 'Profiles from Lines')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('LINES', LINES)
        Tool.Set_Input ('VALUES', VALUES)
        Tool.Set_Output('PROFILE', PROFILE)
        Tool.Set_Output('PROFILES', PROFILES)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('SPLIT', SPLIT)
        return Tool.Execute(Verbose)
    return False

def Profile_from_Points(
    GRID=None,
    TABLE=None,
    VALUES=None,
    RESULT=None,
    X=None,
    Y=None, Verbose=2):
    '''
    Profile from Points
    ----------
    [ta_profiles.5]\n
    The tool allows one to query a profile from an input grid (usually a DEM) for point coordinates stored in a table or shapefile. The profile is traced from one point to the next, sampling the grid values along each line segment. Optionally, additional grids can be queried whose values are added to the profile table.\n
    Arguments
    ----------
    - GRID [`input grid`] : Elevation. The input grid to query.
    - TABLE [`input table`] : Coordinates Table. Table with the point coordinates to query.
    - VALUES [`optional input grid list`] : Values. Additional values to be collected along profile.
    - RESULT [`output table`] : Profile. The output table with the queried profile values.
    - X [`table field`] : X Coordinate
    - Y [`table field`] : Y Coordinate

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_profiles', '5', 'Profile from Points')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Input ('TABLE', TABLE)
        Tool.Set_Input ('VALUES', VALUES)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('X', X)
        Tool.Set_Option('Y', Y)
        return Tool.Execute(Verbose)
    return False

def run_tool_ta_profiles_5(
    GRID=None,
    TABLE=None,
    VALUES=None,
    RESULT=None,
    X=None,
    Y=None, Verbose=2):
    '''
    Profile from Points
    ----------
    [ta_profiles.5]\n
    The tool allows one to query a profile from an input grid (usually a DEM) for point coordinates stored in a table or shapefile. The profile is traced from one point to the next, sampling the grid values along each line segment. Optionally, additional grids can be queried whose values are added to the profile table.\n
    Arguments
    ----------
    - GRID [`input grid`] : Elevation. The input grid to query.
    - TABLE [`input table`] : Coordinates Table. Table with the point coordinates to query.
    - VALUES [`optional input grid list`] : Values. Additional values to be collected along profile.
    - RESULT [`output table`] : Profile. The output table with the queried profile values.
    - X [`table field`] : X Coordinate
    - Y [`table field`] : Y Coordinate

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('ta_profiles', '5', 'Profile from Points')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Input ('TABLE', TABLE)
        Tool.Set_Input ('VALUES', VALUES)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('X', X)
        Tool.Set_Option('Y', Y)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

#################################################################################
# MIT License

# Copyright (c) 2023 Olaf Conrad

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#################################################################################

#_________________________________________
##########################################

__bibtex__ = r"""@Article{gmd-8-1991-2015,
AUTHOR  = {Conrad, O. and Bechtel, B. and Bock, M. and Dietrich, H. and Fischer, E. and Gerlitz, L. and Wehberg, J. and Wichmann, V. and B\"ohner, J.},
TITLE   = {System for Automated Geoscientific Analyses (SAGA) v. 2.1.4},
JOURNAL = {Geoscientific Model Development},
VOLUME  = {8},
YEAR    = {2015},
NUMBER  = {7},
PAGES   = {1991--2007},
URL     = {https://gmd.copernicus.org/articles/8/1991/2015/},
DOI     = {10.5194/gmd-8-1991-2015}
}"""


#_________________________________________
##########################################

import os

SAGA_Path = None
if os.name == 'nt': # Windows
	SAGA_Path = os.getenv('SAGA_PATH')
	if not SAGA_Path:
		SAGA_Path = os.path.split(os.path.dirname(__file__))[0]

	if 'add_dll_directory' in dir(os):
		os.add_dll_directory(SAGA_Path)

	os.environ['PATH'] = SAGA_Path + ';' + os.environ['PATH']


#_________________________________________
##########################################

AutoLoadTools = False

from PySAGA import saga_api

saga_api.SG_Initialize_Environment(AutoLoadTools, True, SAGA_Path)


#_________________________________________
##########################################

#_________________________________________
def Version():
	import sys
	print('_______')
	print('Python-' + sys.version)
	print('SAGA-{:s} (loaded {:d} libraries, {:d} tools)'.format(saga_api.SAGA_VERSION,
		saga_api.SG_Get_Tool_Library_Manager().Get_Count(),
		saga_api.SG_Get_Tool_Library_Manager().Get_Tool_Count()
	))
	print('_______\n')
	return True

#_________________________________________
def Summary(LibraryID='', ToolID=''):
	import sys
	print('_______')
	print('Python-' + sys.version)
	print('SAGA-{:s}'.format(saga_api.SAGA_VERSION))
	if not LibraryID:
		print(saga_api.SG_Get_Tool_Library_Manager().Get_Summary(saga_api.SG_SUMMARY_FMT_FLAT).c_str())
	else:
		Library = saga_api.SG_Get_Tool_Library_Manager().Get_Library(LibraryID, True)
		if not Library:
			print('\nError: could not find library with ID \'{:s}\'\n_______\n'.format(LibraryID))
			return False
		if not ToolID:
			Library = saga_api.SG_Get_Tool_Library_Manager().Get_Library(LibraryID, True, saga_api.ESG_Library_Type_Library)
			if Library:
				print(Library.Get_Summary(saga_api.SG_SUMMARY_FMT_FLAT).c_str())
			Library = saga_api.SG_Get_Tool_Library_Manager().Get_Library(LibraryID, True, saga_api.ESG_Library_Type_Chain)
			if Library:
				print(Library.Get_Summary(saga_api.SG_SUMMARY_FMT_FLAT).c_str())
		else:
			Tool = Library.Get_Tool(ToolID)
			if not Tool:
				print('\nError: could not find tool with ID \'{:s}.{:s}\'\n_______\n'.format(LibraryID, ToolID))
				return False
			print(Tool.Get_Summary(True, Tool.Get_MenuPath(True), Tool.Get_Description(), saga_api.SG_SUMMARY_FMT_FLAT).c_str())
	print('_______\n')
	return True


#_________________________________________
##########################################



#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Import/Export
- Name     : Riegl RDB
- ID       : io_riegl_rdb

Description
----------
Tools to import RDB files.
'''

from PySAGA.helper import Tool_Wrapper

def Import_RDB2_Files(
    POINTS=None,
    FILES=None,
    id=None,
    ts=None,
    AMPL=None,
    REFL=None,
    DEV=None,
    pw=None,
    T_IDX=None,
    T_CNT=None,
    CLASS=None,
    RGBA=None,
    EPSG=None, Verbose=2):
    '''
    Import RDB2 Files
    ----------
    [io_riegl_rdb.0]\n
    Import a pointcloud from Riegl RDB 2 format. This is a work in progress.\n
    Arguments
    ----------
    - POINTS [`output point cloud list`] : Point Clouds
    - FILES [`file path`] : Input Files
    - id [`boolean`] : Point Source ID. Default: 0
    - ts [`boolean`] : Time Stamp. Default: 0
    - AMPL [`boolean`] : Amplitude. Default: 0
    - REFL [`boolean`] : Reflectance. Default: 0
    - DEV [`boolean`] : Deviation. Default: 0
    - pw [`boolean`] : Pulse Width. Default: 0
    - T_IDX [`boolean`] : Target Index. Default: 0
    - T_CNT [`boolean`] : Target Count. Default: 0
    - CLASS [`boolean`] : Class. Default: 0
    - RGBA [`boolean`] : RGP Color. Default: 0
    - EPSG [`boolean`] : Load EPSG Geo-Tag. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_riegl_rdb', '0', 'Import RDB2 Files')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILES', FILES)
        Tool.Set_Option('id', id)
        Tool.Set_Option('ts', ts)
        Tool.Set_Option('ampl', AMPL)
        Tool.Set_Option('refl', REFL)
        Tool.Set_Option('dev', DEV)
        Tool.Set_Option('pw', pw)
        Tool.Set_Option('t_idx', T_IDX)
        Tool.Set_Option('t_cnt', T_CNT)
        Tool.Set_Option('class', CLASS)
        Tool.Set_Option('rgba', RGBA)
        Tool.Set_Option('epsg', EPSG)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_riegl_rdb_0(
    POINTS=None,
    FILES=None,
    id=None,
    ts=None,
    AMPL=None,
    REFL=None,
    DEV=None,
    pw=None,
    T_IDX=None,
    T_CNT=None,
    CLASS=None,
    RGBA=None,
    EPSG=None, Verbose=2):
    '''
    Import RDB2 Files
    ----------
    [io_riegl_rdb.0]\n
    Import a pointcloud from Riegl RDB 2 format. This is a work in progress.\n
    Arguments
    ----------
    - POINTS [`output point cloud list`] : Point Clouds
    - FILES [`file path`] : Input Files
    - id [`boolean`] : Point Source ID. Default: 0
    - ts [`boolean`] : Time Stamp. Default: 0
    - AMPL [`boolean`] : Amplitude. Default: 0
    - REFL [`boolean`] : Reflectance. Default: 0
    - DEV [`boolean`] : Deviation. Default: 0
    - pw [`boolean`] : Pulse Width. Default: 0
    - T_IDX [`boolean`] : Target Index. Default: 0
    - T_CNT [`boolean`] : Target Count. Default: 0
    - CLASS [`boolean`] : Class. Default: 0
    - RGBA [`boolean`] : RGP Color. Default: 0
    - EPSG [`boolean`] : Load EPSG Geo-Tag. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_riegl_rdb', '0', 'Import RDB2 Files')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILES', FILES)
        Tool.Set_Option('id', id)
        Tool.Set_Option('ts', ts)
        Tool.Set_Option('ampl', AMPL)
        Tool.Set_Option('refl', REFL)
        Tool.Set_Option('dev', DEV)
        Tool.Set_Option('pw', pw)
        Tool.Set_Option('t_idx', T_IDX)
        Tool.Set_Option('t_cnt', T_CNT)
        Tool.Set_Option('class', CLASS)
        Tool.Set_Option('rgba', RGBA)
        Tool.Set_Option('epsg', EPSG)
        return Tool.Execute(Verbose)
    return False

def Info_about_RDB2_Files(
    FILES=None, Verbose=2):
    '''
    Info about RDB2 Files
    ----------
    [io_riegl_rdb.1]\n
    Print info about a Riegl RDB 2 file. This is a work in progress.\n
    Arguments
    ----------
    - FILES [`file path`] : Input Files

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_riegl_rdb', '1', 'Info about RDB2 Files')
    if Tool.is_Okay():
        Tool.Set_Option('FILES', FILES)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_riegl_rdb_1(
    FILES=None, Verbose=2):
    '''
    Info about RDB2 Files
    ----------
    [io_riegl_rdb.1]\n
    Print info about a Riegl RDB 2 file. This is a work in progress.\n
    Arguments
    ----------
    - FILES [`file path`] : Input Files

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_riegl_rdb', '1', 'Info about RDB2 Files')
    if Tool.is_Okay():
        Tool.Set_Option('FILES', FILES)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Imagery
- Name     : ViGrA [deprecated]
- ID       : imagery_vigra

Description
----------
ViGrA - "Vision with Generic Algorithms"
Version: 1.11.1
ViGrA is a novel computer vision library that puts its main emphasize on customizable algorithms and data structures. By using template techniques similar to those in the C++ Standard Template Library (STL), you can easily adapt any ViGrA component to the needs of your application, without thereby giving up execution speed.
Find out more at the [ViGrA - Homepage](https://ukoethe.github.io/vigra/).
[[Because the ViGrA project seems to be abandoned since quite a while SAGA's ViGrA tool set has been marked as <u>deprecated</u> and might become removed from future SAGA versions!]
'''

from PySAGA.helper import Tool_Wrapper

def Smoothing_ViGrA(
    INPUT=None,
    OUTPUT=None,
    TYPE=None,
    SCALE=None,
    EDGE=None, Verbose=2):
    '''
    Smoothing (ViGrA)
    ----------
    [imagery_vigra.0]\n
    Based on the code example "smooth.cxx" by Ullrich Koethe.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Output
    - TYPE [`choice`] : Type of smoothing. Available Choices: [0] exponential [1] nonlinear [2] gaussian Default: 0
    - SCALE [`floating point number`] : Size of smoothing filter. Minimum: 0.000000 Default: 2.000000 Smoothing kernel size specified as multiple of a cell.
    - EDGE [`floating point number`] : Edge threshold for nonlinear smoothing. Minimum: 0.000000 Default: 1.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '0', 'Smoothing (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('TYPE', TYPE)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('EDGE', EDGE)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_0(
    INPUT=None,
    OUTPUT=None,
    TYPE=None,
    SCALE=None,
    EDGE=None, Verbose=2):
    '''
    Smoothing (ViGrA)
    ----------
    [imagery_vigra.0]\n
    Based on the code example "smooth.cxx" by Ullrich Koethe.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Output
    - TYPE [`choice`] : Type of smoothing. Available Choices: [0] exponential [1] nonlinear [2] gaussian Default: 0
    - SCALE [`floating point number`] : Size of smoothing filter. Minimum: 0.000000 Default: 2.000000 Smoothing kernel size specified as multiple of a cell.
    - EDGE [`floating point number`] : Edge threshold for nonlinear smoothing. Minimum: 0.000000 Default: 1.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '0', 'Smoothing (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('TYPE', TYPE)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('EDGE', EDGE)
        return Tool.Execute(Verbose)
    return False

def Edge_Detection_ViGrA(
    INPUT=None,
    OUTPUT=None,
    TYPE=None,
    SCALE=None,
    THRESHOLD=None, Verbose=2):
    '''
    Edge Detection (ViGrA)
    ----------
    [imagery_vigra.1]\n
    Edge detection.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Edges
    - TYPE [`choice`] : Detector type. Available Choices: [0] Canny [1] Shen-Castan Default: 0
    - SCALE [`floating point number`] : Operator scale. Minimum: 0.000000 Default: 1.000000
    - THRESHOLD [`floating point number`] : Gradient threshold. Minimum: 0.000000 Default: 1.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '1', 'Edge Detection (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('TYPE', TYPE)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_1(
    INPUT=None,
    OUTPUT=None,
    TYPE=None,
    SCALE=None,
    THRESHOLD=None, Verbose=2):
    '''
    Edge Detection (ViGrA)
    ----------
    [imagery_vigra.1]\n
    Edge detection.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Edges
    - TYPE [`choice`] : Detector type. Available Choices: [0] Canny [1] Shen-Castan Default: 0
    - SCALE [`floating point number`] : Operator scale. Minimum: 0.000000 Default: 1.000000
    - THRESHOLD [`floating point number`] : Gradient threshold. Minimum: 0.000000 Default: 1.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '1', 'Edge Detection (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('TYPE', TYPE)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        return Tool.Execute(Verbose)
    return False

def Morphological_Filter_ViGrA(
    INPUT=None,
    OUTPUT=None,
    TYPE=None,
    RADIUS=None,
    RANK=None,
    RESCALE=None, Verbose=2):
    '''
    Morphological Filter (ViGrA)
    ----------
    [imagery_vigra.2]\n
    Morphological filter.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Output
    - TYPE [`choice`] : Operation. Available Choices: [0] Dilation [1] Erosion [2] Median [3] User defined rank Default: 0
    - RADIUS [`integer number`] : Radius (cells). Minimum: 0 Default: 1
    - RANK [`floating point number`] : User defined rank. Minimum: 0.000000 Maximum: 1.000000 Default: 0.500000
    - RESCALE [`boolean`] : Rescale Values (0-255). Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '2', 'Morphological Filter (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('TYPE', TYPE)
        Tool.Set_Option('RADIUS', RADIUS)
        Tool.Set_Option('RANK', RANK)
        Tool.Set_Option('RESCALE', RESCALE)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_2(
    INPUT=None,
    OUTPUT=None,
    TYPE=None,
    RADIUS=None,
    RANK=None,
    RESCALE=None, Verbose=2):
    '''
    Morphological Filter (ViGrA)
    ----------
    [imagery_vigra.2]\n
    Morphological filter.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Output
    - TYPE [`choice`] : Operation. Available Choices: [0] Dilation [1] Erosion [2] Median [3] User defined rank Default: 0
    - RADIUS [`integer number`] : Radius (cells). Minimum: 0 Default: 1
    - RANK [`floating point number`] : User defined rank. Minimum: 0.000000 Maximum: 1.000000 Default: 0.500000
    - RESCALE [`boolean`] : Rescale Values (0-255). Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '2', 'Morphological Filter (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('TYPE', TYPE)
        Tool.Set_Option('RADIUS', RADIUS)
        Tool.Set_Option('RANK', RANK)
        Tool.Set_Option('RESCALE', RESCALE)
        return Tool.Execute(Verbose)
    return False

def Distance_ViGrA(
    INPUT=None,
    OUTPUT=None,
    NORM=None, Verbose=2):
    '''
    Distance (ViGrA)
    ----------
    [imagery_vigra.3]\n
    Distance to feature cells on a raster. Feature cells are all cells not representing a no-data value.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Features. Features are all pixels not representing a no-data value.
    - OUTPUT [`output grid`] : Distance
    - NORM [`choice`] : Type of distance calculation. Available Choices: [0] Chessboard [1] Manhattan [2] Euclidean Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '3', 'Distance (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('NORM', NORM)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_3(
    INPUT=None,
    OUTPUT=None,
    NORM=None, Verbose=2):
    '''
    Distance (ViGrA)
    ----------
    [imagery_vigra.3]\n
    Distance to feature cells on a raster. Feature cells are all cells not representing a no-data value.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Features. Features are all pixels not representing a no-data value.
    - OUTPUT [`output grid`] : Distance
    - NORM [`choice`] : Type of distance calculation. Available Choices: [0] Chessboard [1] Manhattan [2] Euclidean Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '3', 'Distance (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('NORM', NORM)
        return Tool.Execute(Verbose)
    return False

def Watershed_Segmentation_ViGrA(
    INPUT=None,
    OUTPUT=None,
    SCALE=None,
    RGB=None,
    EDGES=None, Verbose=2):
    '''
    Watershed Segmentation (ViGrA)
    ----------
    [imagery_vigra.4]\n
    Note that the watershed algorithm usually results in an oversegmentation (i.e., too many regions), but its boundary localization is quite good.\n
    Based on the code example "watershed.cxx" by Ullrich Koethe.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Segmentation
    - SCALE [`floating point number`] : Width of gradient filter. Minimum: 0.000000 Default: 1.000000
    - RGB [`boolean`] : RGB coded data. Default: 0
    - EDGES [`boolean`] : Edges. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '4', 'Watershed Segmentation (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('RGB', RGB)
        Tool.Set_Option('EDGES', EDGES)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_4(
    INPUT=None,
    OUTPUT=None,
    SCALE=None,
    RGB=None,
    EDGES=None, Verbose=2):
    '''
    Watershed Segmentation (ViGrA)
    ----------
    [imagery_vigra.4]\n
    Note that the watershed algorithm usually results in an oversegmentation (i.e., too many regions), but its boundary localization is quite good.\n
    Based on the code example "watershed.cxx" by Ullrich Koethe.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Segmentation
    - SCALE [`floating point number`] : Width of gradient filter. Minimum: 0.000000 Default: 1.000000
    - RGB [`boolean`] : RGB coded data. Default: 0
    - EDGES [`boolean`] : Edges. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '4', 'Watershed Segmentation (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('RGB', RGB)
        Tool.Set_Option('EDGES', EDGES)
        return Tool.Execute(Verbose)
    return False

def Fourier_Transform_ViGrA(
    INPUT=None,
    REAL=None,
    IMAG=None,
    CENTER=None, Verbose=2):
    '''
    Fourier Transform (ViGrA)
    ----------
    [imagery_vigra.5]\n
    Fourier Transform.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - REAL [`output grid`] : Real
    - IMAG [`output grid`] : Imaginary
    - CENTER [`boolean`] : Centered. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '5', 'Fourier Transform (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('REAL', REAL)
        Tool.Set_Output('IMAG', IMAG)
        Tool.Set_Option('CENTER', CENTER)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_5(
    INPUT=None,
    REAL=None,
    IMAG=None,
    CENTER=None, Verbose=2):
    '''
    Fourier Transform (ViGrA)
    ----------
    [imagery_vigra.5]\n
    Fourier Transform.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - REAL [`output grid`] : Real
    - IMAG [`output grid`] : Imaginary
    - CENTER [`boolean`] : Centered. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '5', 'Fourier Transform (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('REAL', REAL)
        Tool.Set_Output('IMAG', IMAG)
        Tool.Set_Option('CENTER', CENTER)
        return Tool.Execute(Verbose)
    return False

def Fourier_Transform_Inverse_ViGrA(
    REAL=None,
    IMAG=None,
    OUTPUT=None,
    CENTER=None, Verbose=2):
    '''
    Fourier Transform Inverse (ViGrA)
    ----------
    [imagery_vigra.6]\n
    Inverse Fourier Transform.\n
    Arguments
    ----------
    - REAL [`input grid`] : Real
    - IMAG [`input grid`] : Imaginary
    - OUTPUT [`output grid`] : Output
    - CENTER [`boolean`] : Centered. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '6', 'Fourier Transform Inverse (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('REAL', REAL)
        Tool.Set_Input ('IMAG', IMAG)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('CENTER', CENTER)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_6(
    REAL=None,
    IMAG=None,
    OUTPUT=None,
    CENTER=None, Verbose=2):
    '''
    Fourier Transform Inverse (ViGrA)
    ----------
    [imagery_vigra.6]\n
    Inverse Fourier Transform.\n
    Arguments
    ----------
    - REAL [`input grid`] : Real
    - IMAG [`input grid`] : Imaginary
    - OUTPUT [`output grid`] : Output
    - CENTER [`boolean`] : Centered. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '6', 'Fourier Transform Inverse (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('REAL', REAL)
        Tool.Set_Input ('IMAG', IMAG)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('CENTER', CENTER)
        return Tool.Execute(Verbose)
    return False

def Fourier_Filter_ViGrA(
    INPUT=None,
    OUTPUT=None,
    SCALE=None,
    POWER=None,
    RANGE=None,
    FILTER=None, Verbose=2):
    '''
    Fourier Filter (ViGrA)
    ----------
    [imagery_vigra.8]\n
    Fourier Filter.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Output
    - SCALE [`floating point number`] : Size of smoothing filter. Minimum: 0.000000 Default: 2.000000
    - POWER [`floating point number`] : Power. Default: 0.500000
    - RANGE [`value range`] : Range
    - FILTER [`choice`] : Filter. Available Choices: [0] gaussian [1] power of distance [2] include range [3] exclude range Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '8', 'Fourier Filter (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('POWER', POWER)
        Tool.Set_Option('RANGE', RANGE)
        Tool.Set_Option('FILTER', FILTER)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_8(
    INPUT=None,
    OUTPUT=None,
    SCALE=None,
    POWER=None,
    RANGE=None,
    FILTER=None, Verbose=2):
    '''
    Fourier Filter (ViGrA)
    ----------
    [imagery_vigra.8]\n
    Fourier Filter.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input
    - OUTPUT [`output grid`] : Output
    - SCALE [`floating point number`] : Size of smoothing filter. Minimum: 0.000000 Default: 2.000000
    - POWER [`floating point number`] : Power. Default: 0.500000
    - RANGE [`value range`] : Range
    - FILTER [`choice`] : Filter. Available Choices: [0] gaussian [1] power of distance [2] include range [3] exclude range Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '8', 'Fourier Filter (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('SCALE', SCALE)
        Tool.Set_Option('POWER', POWER)
        Tool.Set_Option('RANGE', RANGE)
        Tool.Set_Option('FILTER', FILTER)
        return Tool.Execute(Verbose)
    return False

def Random_Forest_Classification_ViGrA(
    FEATURES=None,
    TRAINING=None,
    CLASSES=None,
    PROBABILITY=None,
    PROBABILITIES=None,
    IMPORTANCES=None,
    BPROBABILITIES=None,
    FIELD=None,
    LABEL_AS_ID=None,
    DO_MRMR=None,
    MRMR_NFEATURES=None,
    MRMR_DISCRETIZE=None,
    MRMR_THRESHOLD=None,
    MRMR_METHOD=None,
    RF_TREE_COUNT=None,
    RF_TREE_SAMPLES=None,
    RF_REPLACE=None,
    RF_SPLIT_MIN_SIZE=None,
    RF_NODE_FEATURES=None,
    RF_STRATIFICATION=None, Verbose=2):
    '''
    Random Forest Classification (ViGrA)
    ----------
    [imagery_vigra.9]\n
    Random Forest Classification.\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - TRAINING [`input shapes`] : Training Areas
    - CLASSES [`output grid`] : Random Forest Classification
    - PROBABILITY [`output grid`] : Prediction Probability
    - PROBABILITIES [`output grid list`] : Feature Probabilities
    - IMPORTANCES [`output table`] : Feature Importances
    - BPROBABILITIES [`boolean`] : Feature Probabilities. Default: 0
    - FIELD [`table field`] : Label Field
    - LABEL_AS_ID [`boolean`] : Use Label as Identifier. Default: 0 Use training area labels as identifier in classification result, assumes all label values are integer numbers!
    - DO_MRMR [`boolean`] : Minimum Redundancy Feature Selection. Default: 0 Use only features selected by the minimum Redundancy Maximum Relevance (mRMR) algorithm
    - MRMR_NFEATURES [`integer number`] : Number of Features. Minimum: 1 Default: 50
    - MRMR_DISCRETIZE [`boolean`] : Discretization. Default: 1 uncheck this means no discretizaton (i.e. data is already integer)
    - MRMR_THRESHOLD [`floating point number`] : Discretization Threshold. Minimum: 0.000000 Default: 1.000000 a double number of the discretization threshold; set to 0 to make binarization
    - MRMR_METHOD [`choice`] : Selection Method. Available Choices: [0] Mutual Information Difference (MID) [1] Mutual Information Quotient (MIQ) Default: 0
    - RF_TREE_COUNT [`integer number`] : Tree Count. Minimum: 1 Default: 32 How many trees to create?
    - RF_TREE_SAMPLES [`floating point number`] : Samples per Tree. Minimum: 0.000000 Maximum: 1.000000 Default: 1.000000 Specifies the fraction of the total number of samples used per tree for learning.
    - RF_REPLACE [`boolean`] : Sample with Replacement. Default: 1 Sample from training population with or without replacement?
    - RF_SPLIT_MIN_SIZE [`integer number`] : Minimum Node Split Size. Minimum: 1 Default: 1 Number of examples required for a node to be split. Choose 1 for complete growing.
    - RF_NODE_FEATURES [`choice`] : Features per Node. Available Choices: [0] logarithmic [1] square root [2] all Default: 1
    - RF_STRATIFICATION [`choice`] : Stratification. Available Choices: [0] none [1] equal [2] proportional Default: 0 Specifies stratification strategy. Either none, equal amount of class samples, or proportional to fraction of class samples.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '9', 'Random Forest Classification (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Input ('TRAINING', TRAINING)
        Tool.Set_Output('CLASSES', CLASSES)
        Tool.Set_Output('PROBABILITY', PROBABILITY)
        Tool.Set_Output('PROBABILITIES', PROBABILITIES)
        Tool.Set_Output('IMPORTANCES', IMPORTANCES)
        Tool.Set_Option('BPROBABILITIES', BPROBABILITIES)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('LABEL_AS_ID', LABEL_AS_ID)
        Tool.Set_Option('DO_MRMR', DO_MRMR)
        Tool.Set_Option('mRMR_NFEATURES', MRMR_NFEATURES)
        Tool.Set_Option('mRMR_DISCRETIZE', MRMR_DISCRETIZE)
        Tool.Set_Option('mRMR_THRESHOLD', MRMR_THRESHOLD)
        Tool.Set_Option('mRMR_METHOD', MRMR_METHOD)
        Tool.Set_Option('RF_TREE_COUNT', RF_TREE_COUNT)
        Tool.Set_Option('RF_TREE_SAMPLES', RF_TREE_SAMPLES)
        Tool.Set_Option('RF_REPLACE', RF_REPLACE)
        Tool.Set_Option('RF_SPLIT_MIN_SIZE', RF_SPLIT_MIN_SIZE)
        Tool.Set_Option('RF_NODE_FEATURES', RF_NODE_FEATURES)
        Tool.Set_Option('RF_STRATIFICATION', RF_STRATIFICATION)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_9(
    FEATURES=None,
    TRAINING=None,
    CLASSES=None,
    PROBABILITY=None,
    PROBABILITIES=None,
    IMPORTANCES=None,
    BPROBABILITIES=None,
    FIELD=None,
    LABEL_AS_ID=None,
    DO_MRMR=None,
    MRMR_NFEATURES=None,
    MRMR_DISCRETIZE=None,
    MRMR_THRESHOLD=None,
    MRMR_METHOD=None,
    RF_TREE_COUNT=None,
    RF_TREE_SAMPLES=None,
    RF_REPLACE=None,
    RF_SPLIT_MIN_SIZE=None,
    RF_NODE_FEATURES=None,
    RF_STRATIFICATION=None, Verbose=2):
    '''
    Random Forest Classification (ViGrA)
    ----------
    [imagery_vigra.9]\n
    Random Forest Classification.\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - TRAINING [`input shapes`] : Training Areas
    - CLASSES [`output grid`] : Random Forest Classification
    - PROBABILITY [`output grid`] : Prediction Probability
    - PROBABILITIES [`output grid list`] : Feature Probabilities
    - IMPORTANCES [`output table`] : Feature Importances
    - BPROBABILITIES [`boolean`] : Feature Probabilities. Default: 0
    - FIELD [`table field`] : Label Field
    - LABEL_AS_ID [`boolean`] : Use Label as Identifier. Default: 0 Use training area labels as identifier in classification result, assumes all label values are integer numbers!
    - DO_MRMR [`boolean`] : Minimum Redundancy Feature Selection. Default: 0 Use only features selected by the minimum Redundancy Maximum Relevance (mRMR) algorithm
    - MRMR_NFEATURES [`integer number`] : Number of Features. Minimum: 1 Default: 50
    - MRMR_DISCRETIZE [`boolean`] : Discretization. Default: 1 uncheck this means no discretizaton (i.e. data is already integer)
    - MRMR_THRESHOLD [`floating point number`] : Discretization Threshold. Minimum: 0.000000 Default: 1.000000 a double number of the discretization threshold; set to 0 to make binarization
    - MRMR_METHOD [`choice`] : Selection Method. Available Choices: [0] Mutual Information Difference (MID) [1] Mutual Information Quotient (MIQ) Default: 0
    - RF_TREE_COUNT [`integer number`] : Tree Count. Minimum: 1 Default: 32 How many trees to create?
    - RF_TREE_SAMPLES [`floating point number`] : Samples per Tree. Minimum: 0.000000 Maximum: 1.000000 Default: 1.000000 Specifies the fraction of the total number of samples used per tree for learning.
    - RF_REPLACE [`boolean`] : Sample with Replacement. Default: 1 Sample from training population with or without replacement?
    - RF_SPLIT_MIN_SIZE [`integer number`] : Minimum Node Split Size. Minimum: 1 Default: 1 Number of examples required for a node to be split. Choose 1 for complete growing.
    - RF_NODE_FEATURES [`choice`] : Features per Node. Available Choices: [0] logarithmic [1] square root [2] all Default: 1
    - RF_STRATIFICATION [`choice`] : Stratification. Available Choices: [0] none [1] equal [2] proportional Default: 0 Specifies stratification strategy. Either none, equal amount of class samples, or proportional to fraction of class samples.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '9', 'Random Forest Classification (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Input ('TRAINING', TRAINING)
        Tool.Set_Output('CLASSES', CLASSES)
        Tool.Set_Output('PROBABILITY', PROBABILITY)
        Tool.Set_Output('PROBABILITIES', PROBABILITIES)
        Tool.Set_Output('IMPORTANCES', IMPORTANCES)
        Tool.Set_Option('BPROBABILITIES', BPROBABILITIES)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('LABEL_AS_ID', LABEL_AS_ID)
        Tool.Set_Option('DO_MRMR', DO_MRMR)
        Tool.Set_Option('mRMR_NFEATURES', MRMR_NFEATURES)
        Tool.Set_Option('mRMR_DISCRETIZE', MRMR_DISCRETIZE)
        Tool.Set_Option('mRMR_THRESHOLD', MRMR_THRESHOLD)
        Tool.Set_Option('mRMR_METHOD', MRMR_METHOD)
        Tool.Set_Option('RF_TREE_COUNT', RF_TREE_COUNT)
        Tool.Set_Option('RF_TREE_SAMPLES', RF_TREE_SAMPLES)
        Tool.Set_Option('RF_REPLACE', RF_REPLACE)
        Tool.Set_Option('RF_SPLIT_MIN_SIZE', RF_SPLIT_MIN_SIZE)
        Tool.Set_Option('RF_NODE_FEATURES', RF_NODE_FEATURES)
        Tool.Set_Option('RF_STRATIFICATION', RF_STRATIFICATION)
        return Tool.Execute(Verbose)
    return False

def Random_Forest_Presence_Prediction_ViGrA(
    FEATURES=None,
    PRESENCE=None,
    PREDICTION=None,
    PROBABILITY=None,
    BACKGROUND=None,
    DO_MRMR=None,
    MRMR_NFEATURES=None,
    MRMR_DISCRETIZE=None,
    MRMR_THRESHOLD=None,
    MRMR_METHOD=None,
    RF_TREE_COUNT=None,
    RF_TREE_SAMPLES=None,
    RF_REPLACE=None,
    RF_SPLIT_MIN_SIZE=None,
    RF_NODE_FEATURES=None,
    RF_STRATIFICATION=None, Verbose=2):
    '''
    Random Forest Presence Prediction (ViGrA)
    ----------
    [imagery_vigra.10]\n
    Random Forest Presence Prediction\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - PRESENCE [`input shapes`] : Presence Data
    - PREDICTION [`output grid`] : Presence Prediction
    - PROBABILITY [`output grid`] : Presence Probability
    - BACKGROUND [`floating point number`] : Background Sample Density [Percent]. Minimum: 0.000000 Maximum: 100.000000 Default: 1.000000
    - DO_MRMR [`boolean`] : Minimum Redundancy Feature Selection. Default: 0 Use only features selected by the minimum Redundancy Maximum Relevance (mRMR) algorithm
    - MRMR_NFEATURES [`integer number`] : Number of Features. Minimum: 1 Default: 50
    - MRMR_DISCRETIZE [`boolean`] : Discretization. Default: 1 uncheck this means no discretizaton (i.e. data is already integer)
    - MRMR_THRESHOLD [`floating point number`] : Discretization Threshold. Minimum: 0.000000 Default: 1.000000 a double number of the discretization threshold; set to 0 to make binarization
    - MRMR_METHOD [`choice`] : Selection Method. Available Choices: [0] Mutual Information Difference (MID) [1] Mutual Information Quotient (MIQ) Default: 0
    - RF_TREE_COUNT [`integer number`] : Tree Count. Minimum: 1 Default: 32 How many trees to create?
    - RF_TREE_SAMPLES [`floating point number`] : Samples per Tree. Minimum: 0.000000 Maximum: 1.000000 Default: 1.000000 Specifies the fraction of the total number of samples used per tree for learning.
    - RF_REPLACE [`boolean`] : Sample with Replacement. Default: 1 Sample from training population with or without replacement?
    - RF_SPLIT_MIN_SIZE [`integer number`] : Minimum Node Split Size. Minimum: 1 Default: 1 Number of examples required for a node to be split. Choose 1 for complete growing.
    - RF_NODE_FEATURES [`choice`] : Features per Node. Available Choices: [0] logarithmic [1] square root [2] all Default: 1
    - RF_STRATIFICATION [`choice`] : Stratification. Available Choices: [0] none [1] equal [2] proportional Default: 0 Specifies stratification strategy. Either none, equal amount of class samples, or proportional to fraction of class samples.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '10', 'Random Forest Presence Prediction (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Input ('PRESENCE', PRESENCE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('PROBABILITY', PROBABILITY)
        Tool.Set_Option('BACKGROUND', BACKGROUND)
        Tool.Set_Option('DO_MRMR', DO_MRMR)
        Tool.Set_Option('mRMR_NFEATURES', MRMR_NFEATURES)
        Tool.Set_Option('mRMR_DISCRETIZE', MRMR_DISCRETIZE)
        Tool.Set_Option('mRMR_THRESHOLD', MRMR_THRESHOLD)
        Tool.Set_Option('mRMR_METHOD', MRMR_METHOD)
        Tool.Set_Option('RF_TREE_COUNT', RF_TREE_COUNT)
        Tool.Set_Option('RF_TREE_SAMPLES', RF_TREE_SAMPLES)
        Tool.Set_Option('RF_REPLACE', RF_REPLACE)
        Tool.Set_Option('RF_SPLIT_MIN_SIZE', RF_SPLIT_MIN_SIZE)
        Tool.Set_Option('RF_NODE_FEATURES', RF_NODE_FEATURES)
        Tool.Set_Option('RF_STRATIFICATION', RF_STRATIFICATION)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_10(
    FEATURES=None,
    PRESENCE=None,
    PREDICTION=None,
    PROBABILITY=None,
    BACKGROUND=None,
    DO_MRMR=None,
    MRMR_NFEATURES=None,
    MRMR_DISCRETIZE=None,
    MRMR_THRESHOLD=None,
    MRMR_METHOD=None,
    RF_TREE_COUNT=None,
    RF_TREE_SAMPLES=None,
    RF_REPLACE=None,
    RF_SPLIT_MIN_SIZE=None,
    RF_NODE_FEATURES=None,
    RF_STRATIFICATION=None, Verbose=2):
    '''
    Random Forest Presence Prediction (ViGrA)
    ----------
    [imagery_vigra.10]\n
    Random Forest Presence Prediction\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - PRESENCE [`input shapes`] : Presence Data
    - PREDICTION [`output grid`] : Presence Prediction
    - PROBABILITY [`output grid`] : Presence Probability
    - BACKGROUND [`floating point number`] : Background Sample Density [Percent]. Minimum: 0.000000 Maximum: 100.000000 Default: 1.000000
    - DO_MRMR [`boolean`] : Minimum Redundancy Feature Selection. Default: 0 Use only features selected by the minimum Redundancy Maximum Relevance (mRMR) algorithm
    - MRMR_NFEATURES [`integer number`] : Number of Features. Minimum: 1 Default: 50
    - MRMR_DISCRETIZE [`boolean`] : Discretization. Default: 1 uncheck this means no discretizaton (i.e. data is already integer)
    - MRMR_THRESHOLD [`floating point number`] : Discretization Threshold. Minimum: 0.000000 Default: 1.000000 a double number of the discretization threshold; set to 0 to make binarization
    - MRMR_METHOD [`choice`] : Selection Method. Available Choices: [0] Mutual Information Difference (MID) [1] Mutual Information Quotient (MIQ) Default: 0
    - RF_TREE_COUNT [`integer number`] : Tree Count. Minimum: 1 Default: 32 How many trees to create?
    - RF_TREE_SAMPLES [`floating point number`] : Samples per Tree. Minimum: 0.000000 Maximum: 1.000000 Default: 1.000000 Specifies the fraction of the total number of samples used per tree for learning.
    - RF_REPLACE [`boolean`] : Sample with Replacement. Default: 1 Sample from training population with or without replacement?
    - RF_SPLIT_MIN_SIZE [`integer number`] : Minimum Node Split Size. Minimum: 1 Default: 1 Number of examples required for a node to be split. Choose 1 for complete growing.
    - RF_NODE_FEATURES [`choice`] : Features per Node. Available Choices: [0] logarithmic [1] square root [2] all Default: 1
    - RF_STRATIFICATION [`choice`] : Stratification. Available Choices: [0] none [1] equal [2] proportional Default: 0 Specifies stratification strategy. Either none, equal amount of class samples, or proportional to fraction of class samples.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '10', 'Random Forest Presence Prediction (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Input ('PRESENCE', PRESENCE)
        Tool.Set_Output('PREDICTION', PREDICTION)
        Tool.Set_Output('PROBABILITY', PROBABILITY)
        Tool.Set_Option('BACKGROUND', BACKGROUND)
        Tool.Set_Option('DO_MRMR', DO_MRMR)
        Tool.Set_Option('mRMR_NFEATURES', MRMR_NFEATURES)
        Tool.Set_Option('mRMR_DISCRETIZE', MRMR_DISCRETIZE)
        Tool.Set_Option('mRMR_THRESHOLD', MRMR_THRESHOLD)
        Tool.Set_Option('mRMR_METHOD', MRMR_METHOD)
        Tool.Set_Option('RF_TREE_COUNT', RF_TREE_COUNT)
        Tool.Set_Option('RF_TREE_SAMPLES', RF_TREE_SAMPLES)
        Tool.Set_Option('RF_REPLACE', RF_REPLACE)
        Tool.Set_Option('RF_SPLIT_MIN_SIZE', RF_SPLIT_MIN_SIZE)
        Tool.Set_Option('RF_NODE_FEATURES', RF_NODE_FEATURES)
        Tool.Set_Option('RF_STRATIFICATION', RF_STRATIFICATION)
        return Tool.Execute(Verbose)
    return False

def Random_Forest_Table_Classification_ViGrA(
    TABLE=None,
    IMPORTANCES=None,
    FEATURES=None,
    PREDICTION=None,
    TRAINING=None,
    LABEL_AS_ID=None,
    RF_TREE_COUNT=None,
    RF_TREE_SAMPLES=None,
    RF_REPLACE=None,
    RF_SPLIT_MIN_SIZE=None,
    RF_NODE_FEATURES=None,
    RF_STRATIFICATION=None, Verbose=2):
    '''
    Random Forest Table Classification (ViGrA)
    ----------
    [imagery_vigra.11]\n
    Random Forest Table Classification.\n
    Arguments
    ----------
    - TABLE [`input table`] : Table. Table with features, must include class-ID
    - IMPORTANCES [`output table`] : Feature Importances
    - FEATURES [`table fields`] : Features. Select features (table fields) for classification
    - PREDICTION [`table field`] : Prediction. This is field that will have the prediction results. If not set it will be added to the table.
    - TRAINING [`table field`] : Training. this is the table field that defines the training classes
    - LABEL_AS_ID [`boolean`] : Use Label as Identifier. Default: 0 Use training area labels as identifier in classification result, assumes all label values are integer numbers!
    - RF_TREE_COUNT [`integer number`] : Tree Count. Minimum: 1 Default: 32 How many trees to create?
    - RF_TREE_SAMPLES [`floating point number`] : Samples per Tree. Minimum: 0.000000 Maximum: 1.000000 Default: 1.000000 Specifies the fraction of the total number of samples used per tree for learning.
    - RF_REPLACE [`boolean`] : Sample with Replacement. Default: 1 Sample from training population with or without replacement?
    - RF_SPLIT_MIN_SIZE [`integer number`] : Minimum Node Split Size. Minimum: 1 Default: 1 Number of examples required for a node to be split. Choose 1 for complete growing.
    - RF_NODE_FEATURES [`choice`] : Features per Node. Available Choices: [0] logarithmic [1] square root [2] all Default: 1
    - RF_STRATIFICATION [`choice`] : Stratification. Available Choices: [0] none [1] equal [2] proportional Default: 0 Specifies stratification strategy. Either none, equal amount of class samples, or proportional to fraction of class samples.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '11', 'Random Forest Table Classification (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('TABLE', TABLE)
        Tool.Set_Output('IMPORTANCES', IMPORTANCES)
        Tool.Set_Option('FEATURES', FEATURES)
        Tool.Set_Option('PREDICTION', PREDICTION)
        Tool.Set_Option('TRAINING', TRAINING)
        Tool.Set_Option('LABEL_AS_ID', LABEL_AS_ID)
        Tool.Set_Option('RF_TREE_COUNT', RF_TREE_COUNT)
        Tool.Set_Option('RF_TREE_SAMPLES', RF_TREE_SAMPLES)
        Tool.Set_Option('RF_REPLACE', RF_REPLACE)
        Tool.Set_Option('RF_SPLIT_MIN_SIZE', RF_SPLIT_MIN_SIZE)
        Tool.Set_Option('RF_NODE_FEATURES', RF_NODE_FEATURES)
        Tool.Set_Option('RF_STRATIFICATION', RF_STRATIFICATION)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_vigra_11(
    TABLE=None,
    IMPORTANCES=None,
    FEATURES=None,
    PREDICTION=None,
    TRAINING=None,
    LABEL_AS_ID=None,
    RF_TREE_COUNT=None,
    RF_TREE_SAMPLES=None,
    RF_REPLACE=None,
    RF_SPLIT_MIN_SIZE=None,
    RF_NODE_FEATURES=None,
    RF_STRATIFICATION=None, Verbose=2):
    '''
    Random Forest Table Classification (ViGrA)
    ----------
    [imagery_vigra.11]\n
    Random Forest Table Classification.\n
    Arguments
    ----------
    - TABLE [`input table`] : Table. Table with features, must include class-ID
    - IMPORTANCES [`output table`] : Feature Importances
    - FEATURES [`table fields`] : Features. Select features (table fields) for classification
    - PREDICTION [`table field`] : Prediction. This is field that will have the prediction results. If not set it will be added to the table.
    - TRAINING [`table field`] : Training. this is the table field that defines the training classes
    - LABEL_AS_ID [`boolean`] : Use Label as Identifier. Default: 0 Use training area labels as identifier in classification result, assumes all label values are integer numbers!
    - RF_TREE_COUNT [`integer number`] : Tree Count. Minimum: 1 Default: 32 How many trees to create?
    - RF_TREE_SAMPLES [`floating point number`] : Samples per Tree. Minimum: 0.000000 Maximum: 1.000000 Default: 1.000000 Specifies the fraction of the total number of samples used per tree for learning.
    - RF_REPLACE [`boolean`] : Sample with Replacement. Default: 1 Sample from training population with or without replacement?
    - RF_SPLIT_MIN_SIZE [`integer number`] : Minimum Node Split Size. Minimum: 1 Default: 1 Number of examples required for a node to be split. Choose 1 for complete growing.
    - RF_NODE_FEATURES [`choice`] : Features per Node. Available Choices: [0] logarithmic [1] square root [2] all Default: 1
    - RF_STRATIFICATION [`choice`] : Stratification. Available Choices: [0] none [1] equal [2] proportional Default: 0 Specifies stratification strategy. Either none, equal amount of class samples, or proportional to fraction of class samples.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_vigra', '11', 'Random Forest Table Classification (ViGrA)')
    if Tool.is_Okay():
        Tool.Set_Input ('TABLE', TABLE)
        Tool.Set_Output('IMPORTANCES', IMPORTANCES)
        Tool.Set_Option('FEATURES', FEATURES)
        Tool.Set_Option('PREDICTION', PREDICTION)
        Tool.Set_Option('TRAINING', TRAINING)
        Tool.Set_Option('LABEL_AS_ID', LABEL_AS_ID)
        Tool.Set_Option('RF_TREE_COUNT', RF_TREE_COUNT)
        Tool.Set_Option('RF_TREE_SAMPLES', RF_TREE_SAMPLES)
        Tool.Set_Option('RF_REPLACE', RF_REPLACE)
        Tool.Set_Option('RF_SPLIT_MIN_SIZE', RF_SPLIT_MIN_SIZE)
        Tool.Set_Option('RF_NODE_FEATURES', RF_NODE_FEATURES)
        Tool.Set_Option('RF_STRATIFICATION', RF_STRATIFICATION)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Imagery
- Name     : Photogrammetry
- ID       : imagery_photogrammetry

Description
----------
Photogrammetry tools.
'''

from PySAGA.helper import Tool_Wrapper

def Resection_Terrestrial(
    POINTS=None,
    F=None,
    W=None,
    EST_OFFSETS=None,
    PPX=None,
    PPY=None,
    GIVE_DISTORTIONS=None,
    K1=None,
    K2=None,
    K3=None,
    Xc=None,
    Yc=None,
    Zc=None,
    Xt=None,
    Yt=None,
    Zt=None,
    OUTPUT_FILE=None, Verbose=2):
    '''
    Resection (Terrestrial)
    ----------
    [imagery_photogrammetry.0]\n
    Single Image Spatial Resection (Terrestrial): from at least 3 image points with known coordinates, the Cardan angles of the image orientation and the coordinates of the perspective center are calculated by a least-squares adjustment. The Cardan angles refer to the following transformation between image coordinates (x_image) and global coordinates (X_Global):\n
    x_image = R_1(omega) * R_2(kappa) * R_3(alpha) * X_Global\n
    Here R_1, R_2, R_3 denote rotation matrices of a right-handed (passive) coordinate transformation.\n
    The inputs consist of a point cloud containing the identical points with their pixel indices as additional attributes. The origin of pixels is in the lower left corner of the image. The interior orientation parameters of the camera is to be provided. These include Focal Length (mm), Pixel Size (um), Principal Point Offsets (pixels) and optionally the Radial Distortion Parameters. The distortion model being used is as follows:\n
    x_d = x_u (1 - dR)\n
    y_d = y_u (1 - dR)\n
    where, dR = K1 * r_u ^ 2 + K2 * r_u ^ 4 + K3 * r_u ^ 6,\n
    r_u ^ 2  = x_u ^ 2 + y_u ^ 2,\n
    x_u, y_u are the undistorted (corrected) image coordinates in mm,\n
    x_d, y_d are the distorted (observed) image coordinates in mm,\n
    K1 is in [mm ^ -2], K2 is in [mm ^ -4],  K3 is in [mm ^ -6].\n
    Approximate coordinates in [m] for the Projection Center and the center of image are also to be provided.\n
    Following the adjustment the results are written to a text file. The main contents of the text file are: for each iteration the Sum of Squared Residuals, the A-Posteriori Standard Deviation (Sigma Naught) and the Condition of Normal Matrix, and the final estimated exterior orientation parameters (Xc, Yc, Zc, Omega, Kappa, Alpha).\n
    Optionally the Principal Point Offsets can be estimated. This requires at least 4 image points as input.\n
    [Warning] The tool is dedicated to the terrestrial case which means the viewing direction must be sufficiently different from the vertical direction.\n
    The estimation of Principal Point Offsets is not recommended if the condition of the Normal Matrix is less than 10 ^ -7.\n
    Arguments
    ----------
    - POINTS [`input point cloud`] : Measured Points (PC). List of Measured Points as PC
    - F [`floating point number`] : Focal Length (mm). Default: 0.000000 Focal Length in mm
    - W [`floating point number`] : Pixel Width (um). Default: 0.000000 Pixel Width in micro meters
    - EST_OFFSETS [`boolean`] : Estimate Principal Point Offsets?. Default: 0 Do you wish to estimate Principal Point Offsets?
    - PPX [`floating point number`] : Principal Point Offset in X (pixels). Default: 0.000000 Principal Point Offset in X
    - PPY [`floating point number`] : Principal Point Offset in Y (pixels). Default: 0.000000 Principal Point Offset in Y
    - GIVE_DISTORTIONS [`boolean`] : Provide Radial Distortion Parameters?. Default: 0 Do you wish to provide the Radial Distortion Parameters?
    - K1 [`floating point number`] : K1. Default: 0.000000 K1
    - K2 [`floating point number`] : K2. Default: 0.000000 K2
    - K3 [`floating point number`] : K3. Default: 0.000000 K3
    - Xc [`floating point number`] : X. Default: 0.000000 Approximate Coordinate
    - Yc [`floating point number`] : Y. Default: 0.000000 Approximate Coordinate
    - Zc [`floating point number`] : Z. Default: 0.000000 Approximate Coordinate
    - Xt [`floating point number`] : X. Default: 0.000000 Approximate Coordinate
    - Yt [`floating point number`] : Y. Default: 0.000000 Approximate Coordinate
    - Zt [`floating point number`] : Z. Default: 0.000000 Approximate Coordinate
    - OUTPUT_FILE [`file path`] : Output Text File. The file to write the Calculation Results to.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_photogrammetry', '0', 'Resection (Terrestrial)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Option('F', F)
        Tool.Set_Option('W', W)
        Tool.Set_Option('EST_OFFSETS', EST_OFFSETS)
        Tool.Set_Option('ppX', PPX)
        Tool.Set_Option('ppY', PPY)
        Tool.Set_Option('GIVE_DISTORTIONS', GIVE_DISTORTIONS)
        Tool.Set_Option('K1', K1)
        Tool.Set_Option('K2', K2)
        Tool.Set_Option('K3', K3)
        Tool.Set_Option('Xc', Xc)
        Tool.Set_Option('Yc', Yc)
        Tool.Set_Option('Zc', Zc)
        Tool.Set_Option('Xt', Xt)
        Tool.Set_Option('Yt', Yt)
        Tool.Set_Option('Zt', Zt)
        Tool.Set_Option('OUTPUT FILE', OUTPUT_FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_photogrammetry_0(
    POINTS=None,
    F=None,
    W=None,
    EST_OFFSETS=None,
    PPX=None,
    PPY=None,
    GIVE_DISTORTIONS=None,
    K1=None,
    K2=None,
    K3=None,
    Xc=None,
    Yc=None,
    Zc=None,
    Xt=None,
    Yt=None,
    Zt=None,
    OUTPUT_FILE=None, Verbose=2):
    '''
    Resection (Terrestrial)
    ----------
    [imagery_photogrammetry.0]\n
    Single Image Spatial Resection (Terrestrial): from at least 3 image points with known coordinates, the Cardan angles of the image orientation and the coordinates of the perspective center are calculated by a least-squares adjustment. The Cardan angles refer to the following transformation between image coordinates (x_image) and global coordinates (X_Global):\n
    x_image = R_1(omega) * R_2(kappa) * R_3(alpha) * X_Global\n
    Here R_1, R_2, R_3 denote rotation matrices of a right-handed (passive) coordinate transformation.\n
    The inputs consist of a point cloud containing the identical points with their pixel indices as additional attributes. The origin of pixels is in the lower left corner of the image. The interior orientation parameters of the camera is to be provided. These include Focal Length (mm), Pixel Size (um), Principal Point Offsets (pixels) and optionally the Radial Distortion Parameters. The distortion model being used is as follows:\n
    x_d = x_u (1 - dR)\n
    y_d = y_u (1 - dR)\n
    where, dR = K1 * r_u ^ 2 + K2 * r_u ^ 4 + K3 * r_u ^ 6,\n
    r_u ^ 2  = x_u ^ 2 + y_u ^ 2,\n
    x_u, y_u are the undistorted (corrected) image coordinates in mm,\n
    x_d, y_d are the distorted (observed) image coordinates in mm,\n
    K1 is in [mm ^ -2], K2 is in [mm ^ -4],  K3 is in [mm ^ -6].\n
    Approximate coordinates in [m] for the Projection Center and the center of image are also to be provided.\n
    Following the adjustment the results are written to a text file. The main contents of the text file are: for each iteration the Sum of Squared Residuals, the A-Posteriori Standard Deviation (Sigma Naught) and the Condition of Normal Matrix, and the final estimated exterior orientation parameters (Xc, Yc, Zc, Omega, Kappa, Alpha).\n
    Optionally the Principal Point Offsets can be estimated. This requires at least 4 image points as input.\n
    [Warning] The tool is dedicated to the terrestrial case which means the viewing direction must be sufficiently different from the vertical direction.\n
    The estimation of Principal Point Offsets is not recommended if the condition of the Normal Matrix is less than 10 ^ -7.\n
    Arguments
    ----------
    - POINTS [`input point cloud`] : Measured Points (PC). List of Measured Points as PC
    - F [`floating point number`] : Focal Length (mm). Default: 0.000000 Focal Length in mm
    - W [`floating point number`] : Pixel Width (um). Default: 0.000000 Pixel Width in micro meters
    - EST_OFFSETS [`boolean`] : Estimate Principal Point Offsets?. Default: 0 Do you wish to estimate Principal Point Offsets?
    - PPX [`floating point number`] : Principal Point Offset in X (pixels). Default: 0.000000 Principal Point Offset in X
    - PPY [`floating point number`] : Principal Point Offset in Y (pixels). Default: 0.000000 Principal Point Offset in Y
    - GIVE_DISTORTIONS [`boolean`] : Provide Radial Distortion Parameters?. Default: 0 Do you wish to provide the Radial Distortion Parameters?
    - K1 [`floating point number`] : K1. Default: 0.000000 K1
    - K2 [`floating point number`] : K2. Default: 0.000000 K2
    - K3 [`floating point number`] : K3. Default: 0.000000 K3
    - Xc [`floating point number`] : X. Default: 0.000000 Approximate Coordinate
    - Yc [`floating point number`] : Y. Default: 0.000000 Approximate Coordinate
    - Zc [`floating point number`] : Z. Default: 0.000000 Approximate Coordinate
    - Xt [`floating point number`] : X. Default: 0.000000 Approximate Coordinate
    - Yt [`floating point number`] : Y. Default: 0.000000 Approximate Coordinate
    - Zt [`floating point number`] : Z. Default: 0.000000 Approximate Coordinate
    - OUTPUT_FILE [`file path`] : Output Text File. The file to write the Calculation Results to.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_photogrammetry', '0', 'Resection (Terrestrial)')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Option('F', F)
        Tool.Set_Option('W', W)
        Tool.Set_Option('EST_OFFSETS', EST_OFFSETS)
        Tool.Set_Option('ppX', PPX)
        Tool.Set_Option('ppY', PPY)
        Tool.Set_Option('GIVE_DISTORTIONS', GIVE_DISTORTIONS)
        Tool.Set_Option('K1', K1)
        Tool.Set_Option('K2', K2)
        Tool.Set_Option('K3', K3)
        Tool.Set_Option('Xc', Xc)
        Tool.Set_Option('Yc', Yc)
        Tool.Set_Option('Zc', Zc)
        Tool.Set_Option('Xt', Xt)
        Tool.Set_Option('Yt', Yt)
        Tool.Set_Option('Zt', Zt)
        Tool.Set_Option('OUTPUT FILE', OUTPUT_FILE)
        return Tool.Execute(Verbose)
    return False

def Colorisation_PC(
    RGBIMAGE=None,
    PC_IN=None,
    PC_OUT=None,
    RGBIMAGE_GRIDSYSTEM=None,
    GIVE_TIME=None,
    IMG_TIME=None,
    TIME_DIFF=None,
    F=None,
    W=None,
    PPX=None,
    PPY=None,
    GIVE_DISTORTIONS=None,
    K1=None,
    K2=None,
    K3=None,
    Xc=None,
    Yc=None,
    Zc=None,
    OMEGA=None,
    KAPPA=None,
    ALPHA=None, Verbose=2):
    '''
    Colorisation (PC)
    ----------
    [imagery_photogrammetry.1]\n
    This tool attaches the color information from a RGB image to Laser Points.\n
    The RGB Image and the Point Cloud to be colorised are the inputs along with the interior and exterior orientation parameters of the camera. The necessary interior orientation parameters are Focal Length (mm), Pixel Size (um), Principal Point Offsets (pixels) and optionally the Radial Distortion Parameters. The distortion model being used is as follows:\n
    x_d = x_u (1 - dR)\n
    y_d = y_u (1 - dR)\n
    where, dR = K1 * r_u ^ 2 + K2 * r_u ^ 4 + K3 * r_u ^ 6,\n
    r_u ^ 2  = x_u ^ 2 + y_u ^ 2,\n
    x_u, y_u are the undistorted (corrected) image coordinates in mm,\n
    x_d, y_d are the distorted (observed) image coordinates in mm,\n
    K1 is in [mm ^ -2], K2 is in [mm ^ -4],  K3 is in [mm ^ -6].\n
    The necessary exterior orientation parameters are the coordinates of the Projection Center and the Cardan angles Omega, Kappa and Alpha. The Cardan angles refer to the following transformation between image coordinates (x_image) and global coordinates (X_Global):\n
    x_image = R_1(omega) * R_2(kappa) * R_3(alpha) * X_Global\n
    [Optional] If the Time Stamp of the RGB Image is given, an acceptable time difference between the points and the image must provided in order to colorise only those points falling that range. If no time is given all points are colorised.\n
    The result is a colorised point cloud.\n
    Arguments
    ----------
    - RGBIMAGE [`input grid`] : RGB Image. RGB Image
    - PC_IN [`input point cloud`] : Points to be Colorised. Points to be Colorised
    - PC_OUT [`output point cloud`] : Colorised Point Cloud. Colorised Point Cloud
    - RGBIMAGE_GRIDSYSTEM [`grid system`] : Grid system
    - GIVE_TIME [`boolean`] : Provide Time stamp of Image?. Default: 0 Do you wish to provide the time stamp of Image?
    - IMG_TIME [`floating point number`] : Time stamp of Image. Default: 0.000000 Time stamp of Image
    - TIME_DIFF [`floating point number`] : Time Difference between Image & Points. Default: 0.000000 Acceptable Time Difference between Image and Laser Points
    - F [`floating point number`] : Focal Length (mm). Default: 0.000000 Focal Length in mm
    - W [`floating point number`] : Pixel Width (um). Default: 0.000000 Pixel Width in micro meters
    - PPX [`floating point number`] : Principal Point Offset in X (pixles). Default: 0.000000 Principal Point Offset in X
    - PPY [`floating point number`] : Principal Point Offset in Y (pixels). Default: 0.000000 Principal Point Offset in Y
    - GIVE_DISTORTIONS [`boolean`] : Provide Radial Distortion Parameters?. Default: 0 Do you wish to provide the Radial Distortion Parameters?
    - K1 [`floating point number`] : K1. Default: 0.000000 K1
    - K2 [`floating point number`] : K2. Default: 0.000000 K2
    - K3 [`floating point number`] : K3. Default: 0.000000 K3
    - Xc [`floating point number`] : Projection Centre - X. Default: 0.000000 Projection Centre
    - Yc [`floating point number`] : Projection Centre - Y. Default: 0.000000 Projection Centre
    - Zc [`floating point number`] : Projection Centre - Z. Default: 0.000000 Projection Centre
    - OMEGA [`floating point number`] : Omega. Default: 0.000000 Rotation Angle
    - KAPPA [`floating point number`] : Kappa. Default: 0.000000 Rotation Angle
    - ALPHA [`floating point number`] : Alpha. Default: 0.000000 Rotation Angle

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_photogrammetry', '1', 'Colorisation (PC)')
    if Tool.is_Okay():
        Tool.Set_Input ('rgbImage', RGBIMAGE)
        Tool.Set_Input ('PC_IN', PC_IN)
        Tool.Set_Output('PC_OUT', PC_OUT)
        Tool.Set_Option('rgbImage_GRIDSYSTEM', RGBIMAGE_GRIDSYSTEM)
        Tool.Set_Option('GIVE_TIME', GIVE_TIME)
        Tool.Set_Option('IMG_TIME', IMG_TIME)
        Tool.Set_Option('TIME_DIFF', TIME_DIFF)
        Tool.Set_Option('F', F)
        Tool.Set_Option('W', W)
        Tool.Set_Option('ppX', PPX)
        Tool.Set_Option('ppY', PPY)
        Tool.Set_Option('GIVE_DISTORTIONS', GIVE_DISTORTIONS)
        Tool.Set_Option('K1', K1)
        Tool.Set_Option('K2', K2)
        Tool.Set_Option('K3', K3)
        Tool.Set_Option('Xc', Xc)
        Tool.Set_Option('Yc', Yc)
        Tool.Set_Option('Zc', Zc)
        Tool.Set_Option('omega', OMEGA)
        Tool.Set_Option('kappa', KAPPA)
        Tool.Set_Option('alpha', ALPHA)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_photogrammetry_1(
    RGBIMAGE=None,
    PC_IN=None,
    PC_OUT=None,
    RGBIMAGE_GRIDSYSTEM=None,
    GIVE_TIME=None,
    IMG_TIME=None,
    TIME_DIFF=None,
    F=None,
    W=None,
    PPX=None,
    PPY=None,
    GIVE_DISTORTIONS=None,
    K1=None,
    K2=None,
    K3=None,
    Xc=None,
    Yc=None,
    Zc=None,
    OMEGA=None,
    KAPPA=None,
    ALPHA=None, Verbose=2):
    '''
    Colorisation (PC)
    ----------
    [imagery_photogrammetry.1]\n
    This tool attaches the color information from a RGB image to Laser Points.\n
    The RGB Image and the Point Cloud to be colorised are the inputs along with the interior and exterior orientation parameters of the camera. The necessary interior orientation parameters are Focal Length (mm), Pixel Size (um), Principal Point Offsets (pixels) and optionally the Radial Distortion Parameters. The distortion model being used is as follows:\n
    x_d = x_u (1 - dR)\n
    y_d = y_u (1 - dR)\n
    where, dR = K1 * r_u ^ 2 + K2 * r_u ^ 4 + K3 * r_u ^ 6,\n
    r_u ^ 2  = x_u ^ 2 + y_u ^ 2,\n
    x_u, y_u are the undistorted (corrected) image coordinates in mm,\n
    x_d, y_d are the distorted (observed) image coordinates in mm,\n
    K1 is in [mm ^ -2], K2 is in [mm ^ -4],  K3 is in [mm ^ -6].\n
    The necessary exterior orientation parameters are the coordinates of the Projection Center and the Cardan angles Omega, Kappa and Alpha. The Cardan angles refer to the following transformation between image coordinates (x_image) and global coordinates (X_Global):\n
    x_image = R_1(omega) * R_2(kappa) * R_3(alpha) * X_Global\n
    [Optional] If the Time Stamp of the RGB Image is given, an acceptable time difference between the points and the image must provided in order to colorise only those points falling that range. If no time is given all points are colorised.\n
    The result is a colorised point cloud.\n
    Arguments
    ----------
    - RGBIMAGE [`input grid`] : RGB Image. RGB Image
    - PC_IN [`input point cloud`] : Points to be Colorised. Points to be Colorised
    - PC_OUT [`output point cloud`] : Colorised Point Cloud. Colorised Point Cloud
    - RGBIMAGE_GRIDSYSTEM [`grid system`] : Grid system
    - GIVE_TIME [`boolean`] : Provide Time stamp of Image?. Default: 0 Do you wish to provide the time stamp of Image?
    - IMG_TIME [`floating point number`] : Time stamp of Image. Default: 0.000000 Time stamp of Image
    - TIME_DIFF [`floating point number`] : Time Difference between Image & Points. Default: 0.000000 Acceptable Time Difference between Image and Laser Points
    - F [`floating point number`] : Focal Length (mm). Default: 0.000000 Focal Length in mm
    - W [`floating point number`] : Pixel Width (um). Default: 0.000000 Pixel Width in micro meters
    - PPX [`floating point number`] : Principal Point Offset in X (pixles). Default: 0.000000 Principal Point Offset in X
    - PPY [`floating point number`] : Principal Point Offset in Y (pixels). Default: 0.000000 Principal Point Offset in Y
    - GIVE_DISTORTIONS [`boolean`] : Provide Radial Distortion Parameters?. Default: 0 Do you wish to provide the Radial Distortion Parameters?
    - K1 [`floating point number`] : K1. Default: 0.000000 K1
    - K2 [`floating point number`] : K2. Default: 0.000000 K2
    - K3 [`floating point number`] : K3. Default: 0.000000 K3
    - Xc [`floating point number`] : Projection Centre - X. Default: 0.000000 Projection Centre
    - Yc [`floating point number`] : Projection Centre - Y. Default: 0.000000 Projection Centre
    - Zc [`floating point number`] : Projection Centre - Z. Default: 0.000000 Projection Centre
    - OMEGA [`floating point number`] : Omega. Default: 0.000000 Rotation Angle
    - KAPPA [`floating point number`] : Kappa. Default: 0.000000 Rotation Angle
    - ALPHA [`floating point number`] : Alpha. Default: 0.000000 Rotation Angle

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_photogrammetry', '1', 'Colorisation (PC)')
    if Tool.is_Okay():
        Tool.Set_Input ('rgbImage', RGBIMAGE)
        Tool.Set_Input ('PC_IN', PC_IN)
        Tool.Set_Output('PC_OUT', PC_OUT)
        Tool.Set_Option('rgbImage_GRIDSYSTEM', RGBIMAGE_GRIDSYSTEM)
        Tool.Set_Option('GIVE_TIME', GIVE_TIME)
        Tool.Set_Option('IMG_TIME', IMG_TIME)
        Tool.Set_Option('TIME_DIFF', TIME_DIFF)
        Tool.Set_Option('F', F)
        Tool.Set_Option('W', W)
        Tool.Set_Option('ppX', PPX)
        Tool.Set_Option('ppY', PPY)
        Tool.Set_Option('GIVE_DISTORTIONS', GIVE_DISTORTIONS)
        Tool.Set_Option('K1', K1)
        Tool.Set_Option('K2', K2)
        Tool.Set_Option('K3', K3)
        Tool.Set_Option('Xc', Xc)
        Tool.Set_Option('Yc', Yc)
        Tool.Set_Option('Zc', Zc)
        Tool.Set_Option('omega', OMEGA)
        Tool.Set_Option('kappa', KAPPA)
        Tool.Set_Option('alpha', ALPHA)
        return Tool.Execute(Verbose)
    return False

